import sys
from typing import Any, Union, final

if sys.version_info >= (3, 12):
    from collections.abc import Buffer
else:
    from _typeshed import ReadableBuffer as Buffer

def hash(key: Union[bytes, str], seed: int = 0, signed: Any = True) -> int: ...
def hash_from_buffer(
    key: Union[Buffer, str], seed: int = 0, signed: Any = True
) -> int: ...
def hash64(
    key: Union[bytes, str], seed: int = 0, x64arch: Any = True, signed: Any = True
) -> tuple[int, int]: ...
def hash128(
    key: Union[bytes, str], seed: int = 0, x64arch: Any = True, signed: Any = False
) -> int: ...
def hash_bytes(key: Union[bytes, str], seed: int = 0, x64arch: Any = True) -> bytes: ...
def mmh3_32_digest(key: Union[Buffer, str], seed: int = 0) -> bytes: ...
def mmh3_32_sintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_32_uintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_x64_128_digest(key: Union[Buffer, str], seed: int = 0) -> bytes: ...
def mmh3_x64_128_sintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_x64_128_uintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_x64_128_stupledigest(
    key: Union[Buffer, str], seed: int = 0
) -> tuple[int, int]: ...
def mmh3_x64_128_utupledigest(
    key: Union[Buffer, str], seed: int = 0
) -> tuple[int, int]: ...
def mmh3_x86_128_digest(key: Union[Buffer, str], seed: int = 0) -> bytes: ...
def mmh3_x86_128_sintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_x86_128_uintdigest(key: Union[Buffer, str], seed: int = 0) -> int: ...
def mmh3_x86_128_stupledigest(
    key: Union[Buffer, str], seed: int = 0
) -> tuple[int, int]: ...
def mmh3_x86_128_utupledigest(
    key: Union[Buffer, str], seed: int = 0
) -> tuple[int, int]: ...

class Hasher:
    def __init__(self, data: Union[Buffer, None] = None, seed: int = 0) -> None: ...
    def update(self, data: Buffer) -> None: ...
    def digest(self) -> bytes: ...
    def sintdigest(self) -> int: ...
    def uintdigest(self) -> int: ...
    def copy(self) -> Hasher: ...
    @property
    def digest_size(self) -> int: ...
    @property
    def block_size(self) -> int: ...
    @property
    def name(self) -> str: ...

@final
class mmh3_32(Hasher): ...

@final
class mmh3_x64_128(Hasher):
    def stupledigest(self) -> tuple[int, int]: ...
    def utupledigest(self) -> tuple[int, int]: ...

@final
class mmh3_x86_128(Hasher):
    def stupledigest(self) -> tuple[int, int]: ...
    def utupledigest(self) -> tuple[int, int]: ...



# pylint: disable=missing-module-docstring,missing-function-docstring
import mmh3

from helper import u32_to_s32


def test_mmh3_32_digest() -> None:
    hasher = mmh3.mmh3_32()
    hasher.update(b"")
    assert hasher.digest() == b"\x00\x00\x00\x00"

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello, world!")
    assert hasher.digest() == b"\xba\x4c\x88\x24"

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello,")
    hasher.update(b" world!")
    assert hasher.digest() == b"\xba\x4c\x88\x24"

    hasher = mmh3.mmh3_32(b"", 0x9747B28C)
    hasher.update(b"Hello,")
    hasher.update(b" world!")
    assert hasher.digest() == b"\xba\x4c\x88\x24"

    hasher = mmh3.mmh3_32(b"Hello,", 0x9747B28C)
    hasher.update(b" world!")
    assert hasher.digest() == b"\xba\x4c\x88\x24"

    hasher = mmh3.mmh3_32(b"Hello,", seed=0x9747B28C)
    hasher.update(b" world!")
    assert hasher.digest() == b"\xba\x4c\x88\x24"


def test_mmh3_32_sintdigest() -> None:
    hasher = mmh3.mmh3_32()
    hasher.update(b"foo")
    assert hasher.sintdigest() == -156908512

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    hasher = mmh3.mmh3_32()
    hasher.update(b"")
    assert hasher.sintdigest() == 0

    hasher = mmh3.mmh3_32(seed=1)
    hasher.update(b"")
    assert hasher.sintdigest() == 0x514E28B7

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43")
    hasher.update(b"\x65")
    assert hasher.sintdigest() == u32_to_s32(0x7E4A8634)

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43\x65\x87")
    assert hasher.sintdigest() == u32_to_s32(0xF55B516B)

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43")
    hasher.update(b"\x65\x87")
    assert hasher.sintdigest() == u32_to_s32(0xF55B516B)

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello, world!")
    assert hasher.sintdigest() == u32_to_s32(0x24884CBA)

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello,")
    hasher.update(b" world!")
    assert hasher.sintdigest() == u32_to_s32(0x24884CBA)

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"The quick brown fo")
    hasher.update(b"x jumps over the lazy dog")
    assert hasher.sintdigest() == u32_to_s32(0x2FA826CD)


def test_mmh3_32_uintdigest() -> None:
    hasher = mmh3.mmh3_32()
    hasher.update(b"foo")
    assert hasher.uintdigest() == 4138058784

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    hasher = mmh3.mmh3_32()
    hasher.update(b"")
    assert hasher.uintdigest() == 0

    hasher = mmh3.mmh3_32(seed=1)
    hasher.update(b"")
    assert hasher.uintdigest() == 0x514E28B7

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43")
    hasher.update(b"\x65")
    assert hasher.uintdigest() == 0x7E4A8634

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43\x65\x87")
    assert hasher.uintdigest() == 0xF55B516B

    hasher = mmh3.mmh3_32()
    hasher.update(b"\x21\x43")
    hasher.update(b"\x65\x87")
    assert hasher.uintdigest() == 0xF55B516B

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello, world!")
    assert hasher.uintdigest() == 0x24884CBA

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"Hello,")
    hasher.update(b" world!")
    assert hasher.uintdigest() == 0x24884CBA

    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"The quick brown fo")
    hasher.update(b"x jumps over the lazy dog")
    assert hasher.uintdigest() == 0x2FA826CD


def test_mmh3_32_copy() -> None:
    hasher = mmh3.mmh3_32(seed=0x9747B28C)
    hasher.update(b"The quick brown fox")

    hasher2 = hasher.copy()

    hasher.update(b" jumps over the lazy dog")
    assert hasher.uintdigest() == 0x2FA826CD

    hasher2.update(b" jumps over the lazy dog")
    assert hasher2.uintdigest() == 0x2FA826CD


def test_mmh3_x64_128_basic_ops() -> None:
    hasher = mmh3.mmh3_x64_128()
    assert hasher.digest_size == 16
    assert hasher.block_size == 32
    assert hasher.name == "mmh3_x64_128"


def test_mmh3_x64_128_digest() -> None:
    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"foo")
    assert hasher.digest() == b"aE\xf5\x01W\x86q\xe2\x87}\xba+\xe4\x87\xaf~"

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"

    hasher = mmh3.mmh3_x64_128(b"", 0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"

    hasher = mmh3.mmh3_x64_128(b"The quick brown ", seed=0x9747B28C)
    hasher.update(b"fox jumps over the lazy dog")
    assert hasher.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"

    hasher = mmh3.mmh3_x64_128(b"The quick brown ", 0x9747B28C)
    hasher.update(b"fox jumps over the lazy dog")
    assert hasher.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"


def test_mmh3_x64_128_sintdigest() -> None:
    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"")
    assert hasher.sintdigest() == 0

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.sintdigest() == -8943985938913228316176695348732677855

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox j")
    hasher.update(b"umps over the lazy dog")
    assert hasher.sintdigest() == -8943985938913228316176695348732677855


def test_mmh3_x64_128_uintdigest() -> None:
    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"")
    assert hasher.uintdigest() == 0

    hasher = mmh3.mmh3_x64_128(seed=1)
    hasher.update(b"")
    assert hasher.uintdigest() == 108177238965372658051732455265379769525

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"foo")
    assert hasher.uintdigest() == 168394135621993849475852668931176482145

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fo")
    hasher.update(b"o")
    assert hasher.uintdigest() == 168394135621993849475852668931176482145

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fooo")
    assert hasher.uintdigest() == 93757880664175803030724836966881520758

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fooofooo")
    assert hasher.uintdigest() == 211983152696995059280678248292944636041

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fooo")
    hasher.update(b"fooo")
    assert hasher.uintdigest() == 211983152696995059280678248292944636041

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fooofoooo")
    assert hasher.uintdigest() == 338423359992422647011971677127905553798

    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"fooo")
    hasher.update(b"foooo")
    assert hasher.uintdigest() == 338423359992422647011971677127905553798

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.uintdigest() == 331338380982025235147197912083035533601

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"T")
    hasher.update(b"he quick brown fox jumps over the lazy dog")
    assert hasher.uintdigest() == 331338380982025235147197912083035533601

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quic")  # 8 bytes
    hasher.update(b"k brown fox jumps over the lazy dog")
    assert hasher.uintdigest() == 331338380982025235147197912083035533601

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick")
    hasher.update(b" brown fox jumps over the lazy dog")
    assert hasher.uintdigest() == 331338380982025235147197912083035533601


def test_mmh3_x64_128_stupledigest() -> None:
    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"")
    assert hasher.stupledigest() == (0, 0)

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.stupledigest() == (8325606756057297185, -484854449282476315)

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quic")
    hasher.update(b"k brown fox jumps over the lazy dog")
    assert hasher.stupledigest() == (8325606756057297185, -484854449282476315)


def test_mmh3_x64_128_utupledigest() -> None:
    hasher = mmh3.mmh3_x64_128()
    hasher.update(b"")
    assert hasher.utupledigest() == (0, 0)

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.utupledigest() == (8325606756057297185, 17961889624427075301)

    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quic")
    hasher.update(b"k brown fox jumps over the lazy dog")
    assert hasher.utupledigest() == (8325606756057297185, 17961889624427075301)


def test_mmh3_x64_128_copy() -> None:
    hasher = mmh3.mmh3_x64_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox")

    hasher2 = hasher.copy()

    hasher.update(b" jumps over the lazy dog")
    assert hasher.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"

    hasher2.update(b" jumps over the lazy dog")
    assert hasher2.digest() == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"


def test_mmh3_x86_128_basic_ops() -> None:
    hasher = mmh3.mmh3_x86_128()
    assert hasher.digest_size == 16
    assert hasher.block_size == 32
    assert hasher.name == "mmh3_x86_128"


def test_mmh3_x86_128_digest() -> None:
    hasher = mmh3.mmh3_x86_128()
    hasher.update(b"")
    assert (
        hasher.digest()
        == b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    )

    hasher = mmh3.mmh3_x86_128(seed=1)
    hasher.update(b"")
    assert hasher.digest() == b"\xec\xad\xc4\x88\xb9\x01\xd2T\xb9\x01\xd2T\xb9\x01\xd2T"

    hasher = mmh3.mmh3_x86_128()
    hasher.update(b"foo")
    assert hasher.digest() == b"%\x1b|We%\xb6`e%\xb6`e%\xb6`"

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown")  # 15 bytes
    assert (
        hasher.digest() == b"2\xc3\n\xdaW\xc2\xcb\xa9\xc4\xbe\x12\xb9\xdc\x01\xe1\x8e"
    )

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown ")  # 16 bytes
    assert hasher.digest() == b"u\xb6\xf9\x07\xf5|\x93,\x0e\xf5\xf1\xf0k\x98\x83\x19"

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown")  # 15 bytes
    hasher.update(b" fox jumps over the lazy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown ")  # 16 bytes
    hasher.update(b"fox jumps over the lazy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox ju")
    hasher.update(b"mps ove")
    hasher.update(b"r the la")
    hasher.update(b"zy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(b"", 0x9747B28C)
    hasher.update(b"The quick brown fox ju")
    hasher.update(b"mps ove")
    hasher.update(b"r the la")
    hasher.update(b"zy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(b"The quick brown fox ju", seed=0x9747B28C)
    hasher.update(b"mps ove")
    hasher.update(b"r the la")
    hasher.update(b"zy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"

    hasher = mmh3.mmh3_x86_128(b"The quick brown fox ju", 0x9747B28C)
    hasher.update(b"mps ove")
    hasher.update(b"r the la")
    hasher.update(b"zy dog")
    assert hasher.digest() == b"^\xd5\xd4\x8aqa\xb8L\x9c:\xa7\x8e>y\xb6\xcd"


def test_mmh3_x86_128_sintdigest() -> None:
    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.sintdigest() == -66843170628920214366208380873156012706


def test_mmh3_x86_128_uintdigest() -> None:
    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.uintdigest() == 273439196292018249097166226558612198750


def test_mmh3_x86_128_stupledigest() -> None:
    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.stupledigest() == (5528275682885686622, -3623575540584727908)


def test_mmh3_x86_128_utupledigest() -> None:
    hasher = mmh3.mmh3_x86_128(seed=0x9747B28C)
    hasher.update(b"The quick brown fox jumps over the lazy dog")
    assert hasher.utupledigest() == (5528275682885686622, 14823168533124823708)



# pylint: disable=missing-module-docstring,missing-function-docstring
import mmh3


def test_function_docstrings() -> None:
    assert "__doc__" in dir(mmh3.hash)
    assert mmh3.hash.__doc__ is not None
    assert mmh3.hash.__doc__.startswith("hash(key, seed=0, signed=True) -> int\n\n")

    assert "__doc__" in dir(mmh3.hash_from_buffer)
    assert mmh3.hash_from_buffer.__doc__ is not None
    assert mmh3.hash_from_buffer.__doc__.startswith(
        "hash_from_buffer(key, seed=0, signed=True) -> int\n\n"
    )

    assert "__doc__" in dir(mmh3.hash64)
    assert mmh3.hash64.__doc__ is not None
    assert mmh3.hash64.__doc__.startswith(
        "hash64(key, seed=0, x64arch=True, signed=True) -> tuple[int, int]\n\n"
    )

    assert "__doc__" in dir(mmh3.hash128)
    assert mmh3.hash128.__doc__ is not None
    assert mmh3.hash128.__doc__.startswith(
        "hash128(key, seed=0, x64arch=True, signed=False) -> int\n\n"
    )

    assert "__doc__" in dir(mmh3.hash_bytes)
    assert mmh3.hash_bytes.__doc__ is not None
    assert mmh3.hash_bytes.__doc__.startswith(
        "hash_bytes(key, seed=0, x64arch=True) -> bytes\n\n"
    )


def test_module_docstring() -> None:
    assert "__doc__" in dir(mmh3)
    assert mmh3.__doc__ is not None
    assert mmh3.__doc__.startswith("A Python front-end to MurmurHash3")



# pylint: disable=missing-module-docstring,missing-function-docstring
import sys

import mmh3

from helper import u32_to_s32


def test_hash() -> None:
    assert mmh3.hash("foo") == -156908512

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    assert mmh3.hash(b"", seed=0) == 0
    assert mmh3.hash(b"", seed=1) == 0x514E28B7
    assert mmh3.hash(b"", seed=0xFFFFFFFF) == u32_to_s32(0x81F16F39)
    assert mmh3.hash(b"\x21\x43\x65\x87", 0) == u32_to_s32(0xF55B516B)
    assert mmh3.hash(b"\x21\x43\x65\x87", 0x5082EDEE) == u32_to_s32(0x2362F9DE)
    assert mmh3.hash(b"\x21\x43\x65", 0) == u32_to_s32(0x7E4A8634)
    assert mmh3.hash(b"\x21\x43", 0) == u32_to_s32(0xA0F7B07A)
    assert mmh3.hash(b"\x21", 0) == u32_to_s32(0x72661CF4)
    assert mmh3.hash(b"\xff\xff\xff\xff", 0) == u32_to_s32(0x76293B50)
    assert mmh3.hash(b"\x00\x00\x00\x00", 0) == u32_to_s32(0x2362F9DE)
    assert mmh3.hash(b"\x00\x00\x00", 0) == u32_to_s32(0x85F0B427)
    assert mmh3.hash(b"\x00\x00", 0) == u32_to_s32(0x30F4C306)
    assert mmh3.hash(b"\x00", 0) == u32_to_s32(0x514E28B7)

    assert mmh3.hash("aaaa", 0x9747B28C) == u32_to_s32(0x5A97808A)
    assert mmh3.hash("aaa", 0x9747B28C) == u32_to_s32(0x283E0130)
    assert mmh3.hash("aa", 0x9747B28C) == u32_to_s32(0x5D211726)
    assert mmh3.hash("a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.hash("abcd", 0x9747B28C) == u32_to_s32(0xF0478627)
    assert mmh3.hash("abc", 0x9747B28C) == u32_to_s32(0xC84A62DD)
    assert mmh3.hash("ab", 0x9747B28C) == u32_to_s32(0x74875592)
    assert mmh3.hash("a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.hash("Hello, world!", 0x9747B28C) == u32_to_s32(0x24884CBA)

    assert mmh3.hash("ππππππππ".encode("utf-8"), 0x9747B28C) == u32_to_s32(0xD58063C1)

    assert mmh3.hash("a" * 256, 0x9747B28C) == u32_to_s32(0x37405BDC)

    assert mmh3.hash("abc", 0) == u32_to_s32(0xB3DD93FA)
    assert mmh3.hash(
        "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0
    ) == u32_to_s32(0xEE925B90)

    assert mmh3.hash(
        "The quick brown fox jumps over the lazy dog", 0x9747B28C
    ) == u32_to_s32(0x2FA826CD)


def test_hash_unsigned() -> None:
    assert mmh3.hash("foo", signed=False) == 4138058784

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    assert mmh3.hash(b"", seed=0, signed=False) == 0
    assert mmh3.hash(b"", seed=1, signed=False) == 0x514E28B7
    assert mmh3.hash(b"", seed=0xFFFFFFFF, signed=False) == 0x81F16F39
    assert mmh3.hash(b"\x21\x43\x65\x87", 0, signed=False) == 0xF55B516B
    assert mmh3.hash(b"\x21\x43\x65\x87", 0x5082EDEE, signed=False) == 0x2362F9DE
    assert mmh3.hash(b"\x21\x43\x65", 0, signed=False) == 0x7E4A8634
    assert mmh3.hash(b"\x21\x43", 0, signed=False) == 0xA0F7B07A
    assert mmh3.hash(b"\x21", 0, signed=False) == 0x72661CF4
    assert mmh3.hash(b"\xff\xff\xff\xff", 0, signed=False) == 0x76293B50
    assert mmh3.hash(b"\x00\x00\x00\x00", 0, signed=False) == 0x2362F9DE
    assert mmh3.hash(b"\x00\x00\x00", 0, signed=False) == 0x85F0B427
    assert mmh3.hash(b"\x00\x00", 0, signed=False) == 0x30F4C306
    assert mmh3.hash(b"\x00", 0, signed=False) == 0x514E28B7

    assert mmh3.hash("aaaa", 0x9747B28C, signed=False) == 0x5A97808A
    assert mmh3.hash("aaa", 0x9747B28C, signed=False) == 0x283E0130
    assert mmh3.hash("aa", 0x9747B28C, signed=False) == 0x5D211726
    assert mmh3.hash("a", 0x9747B28C, signed=False) == 0x7FA09EA6

    assert mmh3.hash("abcd", 0x9747B28C, signed=False) == 0xF0478627
    assert mmh3.hash("abc", 0x9747B28C, signed=False) == 0xC84A62DD
    assert mmh3.hash("ab", 0x9747B28C, signed=False) == 0x74875592
    assert mmh3.hash("a", 0x9747B28C, signed=False) == 0x7FA09EA6

    assert mmh3.hash("Hello, world!", 0x9747B28C, signed=False) == 0x24884CBA

    assert mmh3.hash("ππππππππ".encode("utf-8"), 0x9747B28C, signed=False) == 0xD58063C1

    assert mmh3.hash("a" * 256, 0x9747B28C, signed=False) == 0x37405BDC

    assert mmh3.hash("abc", 0, signed=False) == 0xB3DD93FA
    assert (
        mmh3.hash(
            "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0, signed=False
        )
        == 0xEE925B90
    )

    assert (
        mmh3.hash(
            "The quick brown fox jumps over the lazy dog", 0x9747B28C, signed=False
        )
        == 0x2FA826CD
    )

    assert (
        mmh3.hash(
            "The quick brown fox jumps over the lazy dog", 0x9747B28C, signed=False
        )
        == 0x2FA826CD
    )


def test_hash2() -> None:
    assert mmh3.hash("foo") == -156908512

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    assert mmh3.hash(b"", seed=0) == 0
    assert mmh3.hash(b"", seed=1) == 0x514E28B7
    assert mmh3.hash(b"", seed=0xFFFFFFFF) == u32_to_s32(0x81F16F39)
    assert mmh3.hash(b"\x21\x43\x65\x87", 0) == u32_to_s32(0xF55B516B)
    assert mmh3.hash(b"\x21\x43\x65\x87", 0x5082EDEE) == u32_to_s32(0x2362F9DE)
    assert mmh3.hash(b"\x21\x43\x65", 0) == u32_to_s32(0x7E4A8634)
    assert mmh3.hash(b"\x21\x43", 0) == u32_to_s32(0xA0F7B07A)
    assert mmh3.hash(b"\x21", 0) == u32_to_s32(0x72661CF4)
    assert mmh3.hash(b"\xff\xff\xff\xff", 0) == u32_to_s32(0x76293B50)
    assert mmh3.hash(b"\x00\x00\x00\x00", 0) == u32_to_s32(0x2362F9DE)
    assert mmh3.hash(b"\x00\x00\x00", 0) == u32_to_s32(0x85F0B427)
    assert mmh3.hash(b"\x00\x00", 0) == u32_to_s32(0x30F4C306)
    assert mmh3.hash(b"\x00", 0) == u32_to_s32(0x514E28B7)

    assert mmh3.hash("aaaa", 0x9747B28C) == u32_to_s32(0x5A97808A)
    assert mmh3.hash("aaa", 0x9747B28C) == u32_to_s32(0x283E0130)
    assert mmh3.hash("aa", 0x9747B28C) == u32_to_s32(0x5D211726)
    assert mmh3.hash("a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.hash("abcd", 0x9747B28C) == u32_to_s32(0xF0478627)
    assert mmh3.hash("abc", 0x9747B28C) == u32_to_s32(0xC84A62DD)
    assert mmh3.hash("ab", 0x9747B28C) == u32_to_s32(0x74875592)
    assert mmh3.hash("a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.hash("Hello, world!", 0x9747B28C) == u32_to_s32(0x24884CBA)

    assert mmh3.hash("ππππππππ".encode("utf-8"), 0x9747B28C) == u32_to_s32(0xD58063C1)

    assert mmh3.hash("a" * 256, 0x9747B28C) == u32_to_s32(0x37405BDC)

    assert mmh3.hash("abc", 0) == u32_to_s32(0xB3DD93FA)
    assert mmh3.hash(
        "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0
    ) == u32_to_s32(0xEE925B90)

    assert mmh3.hash(
        "The quick brown fox jumps over the lazy dog", 0x9747B28C
    ) == u32_to_s32(0x2FA826CD)


def test_hash_from_buffer() -> None:
    mview = memoryview("foo".encode("utf8"))
    assert mmh3.hash_from_buffer(mview) == -156908512
    assert mmh3.hash_from_buffer(mview, signed=False) == 4138058784


def test_hash_bytes() -> None:
    assert mmh3.hash_bytes("foo") == b"aE\xf5\x01W\x86q\xe2\x87}\xba+\xe4\x87\xaf~"
    assert (
        mmh3.hash_bytes("foo", 0, True)
        == b"aE\xf5\x01W\x86q\xe2\x87}\xba+\xe4\x87\xaf~"
    )

    # Test vectors from https://github.com/PeterScott/murmur3/blob/master/test.c
    assert mmh3.hash_bytes("Hello, world!", 123, x64arch=False) == (
        0x9E37C886A41621625A1AACD761C9129E
    ).to_bytes(16, "little")
    assert mmh3.hash_bytes("", 123, x64arch=False) == (
        0x26F3E79926F3E79926F3E799FEDC5245
    ).to_bytes(16, "little")


def test_hash64() -> None:
    assert mmh3.hash64("foo") == (-2129773440516405919, 9128664383759220103)
    assert mmh3.hash64("foo", signed=False) == (
        16316970633193145697,
        9128664383759220103,
    )

    assert mmh3.hash64("The quick brown fox jumps over the lazy dog", 0x9747B28C) == (
        8325606756057297185,
        -484854449282476315,
    )
    assert mmh3.hash64(
        "The quick brown fox jumps over the lazy dog", 0x9747B28C, signed=False
    ) == (
        8325606756057297185,
        17961889624427075301,
    )
    assert mmh3.hash64("foo", signed=False, x64arch=True) == (
        16316970633193145697,
        9128664383759220103,
    )

    # Test vectors from https://github.com/PeterScott/murmur3/blob/master/test.c
    assert mmh3.hash64("Hello, world!", 123, signed=False, x64arch=False) == (
        0x5A1AACD761C9129E,
        0x9E37C886A4162162,
    )

    assert mmh3.hash64("", 123, False, False) == (
        0x26F3E799FEDC5245,
        0x26F3E79926F3E799,
    )


def test_hash128() -> None:
    assert mmh3.hash128("foo") == 168394135621993849475852668931176482145
    assert mmh3.hash128("foo", 42) == 215966891540331383248189432718888555506
    assert (
        mmh3.hash128("foo", 42, signed=False) == 215966891540331383248189432718888555506
    )
    assert (
        mmh3.hash128("foo", 42, signed=True) == -124315475380607080215185174712879655950
    )
    # Test vectors from https://github.com/PeterScott/murmur3/blob/master/test.c
    assert (
        mmh3.hash128("Hello, world!", 123, signed=False, x64arch=False)
        == 0x9E37C886A41621625A1AACD761C9129E
    )
    assert mmh3.hash128("", 123, False, False) == 0x26F3E79926F3E79926F3E799FEDC5245


def test_mmh3_32_digest() -> None:
    assert mmh3.mmh3_32_digest(b"") == b"\0\0\0\0"
    assert mmh3.mmh3_32_digest(b"", 0) == b"\0\0\0\0"
    assert mmh3.mmh3_32_digest(b"\x21\x43\x65\x87", 0) == (0xF55B516B).to_bytes(
        4, "little"
    )
    assert mmh3.mmh3_32_digest(b"\x21\x43\x65\x87", u32_to_s32(0x5082EDEE)) == (
        0x2362F9DE
    ).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\x21\x43\x65", 0) == (0x7E4A8634).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\x21\x43", 0) == (0xA0F7B07A).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\x21", 0) == (0x72661CF4).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\xff\xff\xff\xff", 0) == (0x76293B50).to_bytes(
        4, "little"
    )
    assert mmh3.mmh3_32_digest(b"\x00\x00\x00\x00", 0) == (0x2362F9DE).to_bytes(
        4, "little"
    )
    assert mmh3.mmh3_32_digest(b"\x00\x00\x00", 0) == (0x85F0B427).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\x00\x00", 0) == (0x30F4C306).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"\x00", 0) == (0x514E28B7).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(b"aaaa", 0x9747B28C) == (0x5A97808A).to_bytes(
        4, "little"
    )
    assert mmh3.mmh3_32_digest(b"aaa", 0x9747B28C) == (0x283E0130).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"aa", 0x9747B28C) == (0x5D211726).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"a", 0x9747B28C) == (0x7FA09EA6).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(b"abcd", 0x9747B28C) == (0xF0478627).to_bytes(
        4, "little"
    )
    assert mmh3.mmh3_32_digest(b"abc", 0x9747B28C) == (0xC84A62DD).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"ab", 0x9747B28C) == (0x74875592).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(b"a", 0x9747B28C) == (0x7FA09EA6).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(b"Hello, world!", 0x9747B28C) == (0x24884CBA).to_bytes(
        4, "little"
    )

    assert mmh3.mmh3_32_digest("ππππππππ".encode("utf-8"), 0x9747B28C) == (
        0xD58063C1
    ).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(b"a" * 256, 0x9747B28C) == (0x37405BDC).to_bytes(
        4, "little"
    )

    assert mmh3.mmh3_32_digest(b"abc", 0) == (0xB3DD93FA).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(
        b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0
    ) == (0xEE925B90).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(
        b"The quick brown fox jumps over the lazy dog", 0x9747B28C
    ) == (0x2FA826CD).to_bytes(4, "little")

    assert mmh3.mmh3_32_digest(bytearray(b"aaaa"), 0x9747B28C) == (0x5A97808A).to_bytes(
        4, "little"
    )
    v = memoryview(b"aaaa")
    assert mmh3.mmh3_32_digest(v, 0x9747B28C) == (0x5A97808A).to_bytes(4, "little")
    assert mmh3.mmh3_32_digest(v[1:3], 0x9747B28C) == (0x5D211726).to_bytes(4, "little")


def test_mmh3_sintdigest() -> None:
    assert mmh3.mmh3_32_sintdigest(b"foo") == -156908512
    assert mmh3.mmh3_32_sintdigest(bytearray(b"foo")) == -156908512
    assert mmh3.mmh3_32_sintdigest(memoryview(b"foobar")[0:3]) == -156908512

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    assert mmh3.mmh3_32_sintdigest(b"", 0) == 0
    assert mmh3.mmh3_32_sintdigest(b"", 1) == 0x514E28B7
    assert mmh3.mmh3_32_sintdigest(b"", 0xFFFFFFFF) == u32_to_s32(0x81F16F39)
    assert mmh3.mmh3_32_sintdigest(b"\x21\x43\x65\x87", 0) == u32_to_s32(0xF55B516B)
    assert mmh3.mmh3_32_sintdigest(
        b"\x21\x43\x65\x87", u32_to_s32(0x5082EDEE)
    ) == u32_to_s32(0x2362F9DE)
    assert mmh3.mmh3_32_sintdigest(b"\x21\x43\x65", 0) == u32_to_s32(0x7E4A8634)
    assert mmh3.mmh3_32_sintdigest(b"\x21\x43", 0) == u32_to_s32(0xA0F7B07A)
    assert mmh3.mmh3_32_sintdigest(b"\x21", 0) == u32_to_s32(0x72661CF4)
    assert mmh3.mmh3_32_sintdigest(b"\xff\xff\xff\xff", 0) == u32_to_s32(0x76293B50)
    assert mmh3.mmh3_32_sintdigest(b"\x00\x00\x00\x00", 0) == u32_to_s32(0x2362F9DE)
    assert mmh3.mmh3_32_sintdigest(b"\x00\x00\x00", 0) == u32_to_s32(0x85F0B427)
    assert mmh3.mmh3_32_sintdigest(b"\x00\x00", 0) == u32_to_s32(0x30F4C306)
    assert mmh3.mmh3_32_sintdigest(b"\x00", 0) == u32_to_s32(0x514E28B7)

    assert mmh3.mmh3_32_sintdigest(b"aaaa", 0x9747B28C) == u32_to_s32(0x5A97808A)
    assert mmh3.mmh3_32_sintdigest(b"aaa", 0x9747B28C) == u32_to_s32(0x283E0130)
    assert mmh3.mmh3_32_sintdigest(b"aa", 0x9747B28C) == u32_to_s32(0x5D211726)
    assert mmh3.mmh3_32_sintdigest(b"a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.mmh3_32_sintdigest(b"abcd", 0x9747B28C) == u32_to_s32(0xF0478627)
    assert mmh3.mmh3_32_sintdigest(b"abc", 0x9747B28C) == u32_to_s32(0xC84A62DD)
    assert mmh3.mmh3_32_sintdigest(b"ab", 0x9747B28C) == u32_to_s32(0x74875592)
    assert mmh3.mmh3_32_sintdigest(b"a", 0x9747B28C) == u32_to_s32(0x7FA09EA6)

    assert mmh3.mmh3_32_sintdigest(b"Hello, world!", 0x9747B28C) == u32_to_s32(
        0x24884CBA
    )

    assert mmh3.mmh3_32_sintdigest(
        "ππππππππ".encode("utf-8"), 0x9747B28C
    ) == u32_to_s32(0xD58063C1)

    assert mmh3.mmh3_32_sintdigest(b"a" * 256, 0x9747B28C) == u32_to_s32(0x37405BDC)

    assert mmh3.mmh3_32_sintdigest(b"abc", 0) == u32_to_s32(0xB3DD93FA)
    assert mmh3.mmh3_32_sintdigest(
        b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0
    ) == u32_to_s32(0xEE925B90)

    assert mmh3.mmh3_32_sintdigest(
        b"The quick brown fox jumps over the lazy dog", 0x9747B28C
    ) == u32_to_s32(0x2FA826CD)


def test_mmh3_uintdigest() -> None:
    assert mmh3.mmh3_32_uintdigest(b"foo") == 4138058784
    assert mmh3.mmh3_32_uintdigest(bytearray(b"foo")) == 4138058784
    assert mmh3.mmh3_32_uintdigest(memoryview(b"foobar")[0:3]) == 4138058784

    # Test vectors devised by Ian Boyd
    # https://stackoverflow.com/a/31929528
    assert mmh3.mmh3_32_uintdigest(b"") == 0
    assert mmh3.mmh3_32_uintdigest(b"", 0) == 0
    assert mmh3.mmh3_32_uintdigest(b"", 1) == 0x514E28B7
    assert mmh3.mmh3_32_uintdigest(b"", 0xFFFFFFFF) == 0x81F16F39
    assert mmh3.mmh3_32_uintdigest(b"\x21\x43\x65\x87", 0) == 0xF55B516B
    assert mmh3.mmh3_32_uintdigest(b"\x21\x43\x65\x87", 0x5082EDEE) == 0x2362F9DE
    assert mmh3.mmh3_32_uintdigest(b"\x21\x43\x65", 0) == 0x7E4A8634
    assert mmh3.mmh3_32_uintdigest(b"\x21\x43", 0) == 0xA0F7B07A
    assert mmh3.mmh3_32_uintdigest(b"\x21", 0) == 0x72661CF4
    assert mmh3.mmh3_32_uintdigest(b"\xff\xff\xff\xff", 0) == 0x76293B50
    assert mmh3.mmh3_32_uintdigest(b"\x00\x00\x00\x00", 0) == 0x2362F9DE
    assert mmh3.mmh3_32_uintdigest(b"\x00\x00\x00", 0) == 0x85F0B427
    assert mmh3.mmh3_32_uintdigest(b"\x00\x00", 0) == 0x30F4C306
    assert mmh3.mmh3_32_uintdigest(b"\x00", 0) == 0x514E28B7

    assert mmh3.mmh3_32_uintdigest(b"aaaa", 0x9747B28C) == 0x5A97808A
    assert mmh3.mmh3_32_uintdigest(b"aaa", 0x9747B28C) == 0x283E0130
    assert mmh3.mmh3_32_uintdigest(b"aa", 0x9747B28C) == 0x5D211726
    assert mmh3.mmh3_32_uintdigest(b"a", 0x9747B28C) == 0x7FA09EA6

    assert mmh3.mmh3_32_uintdigest(b"abcd", 0x9747B28C) == 0xF0478627
    assert mmh3.mmh3_32_uintdigest(b"abc", 0x9747B28C) == 0xC84A62DD
    assert mmh3.mmh3_32_uintdigest(b"ab", 0x9747B28C) == 0x74875592
    assert mmh3.mmh3_32_uintdigest(b"a", 0x9747B28C) == 0x7FA09EA6

    assert mmh3.mmh3_32_uintdigest(b"Hello, world!", 0x9747B28C) == 0x24884CBA

    assert mmh3.mmh3_32_uintdigest("ππππππππ".encode("utf-8"), 0x9747B28C) == 0xD58063C1

    assert mmh3.mmh3_32_uintdigest(b"a" * 256, 0x9747B28C) == 0x37405BDC

    assert mmh3.mmh3_32_uintdigest(b"abc", 0) == 0xB3DD93FA
    assert (
        mmh3.mmh3_32_uintdigest(
            b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", 0
        )
        == 0xEE925B90
    )

    assert (
        mmh3.mmh3_32_uintdigest(
            b"The quick brown fox jumps over the lazy dog", 0x9747B28C
        )
        == 0x2FA826CD
    )

    assert (
        mmh3.mmh3_32_uintdigest(
            b"The quick brown fox jumps over the lazy dog", 0x9747B28C
        )
        == 0x2FA826CD
    )


def test_mmh3_x64_128_digest() -> None:
    assert (
        mmh3.mmh3_x64_128_digest(b"foo")
        == b"aE\xf5\x01W\x86q\xe2\x87}\xba+\xe4\x87\xaf~"
    )

    assert (
        mmh3.mmh3_x64_128_digest(
            b"The quick brown fox jumps over the lazy dog", 0x9747B28C
        )
        == b"!1c\xd2;\x7f\x8as\xe5\x16\xc0~rsE\xf9"
    )

    v = bytearray(b"bar boo bar")
    mv = memoryview(v)
    v[4] = ord("f")

    assert (
        mmh3.mmh3_x64_128_digest(mv[4:7])
        == b"aE\xf5\x01W\x86q\xe2\x87}\xba+\xe4\x87\xaf~"
    )


def test_mmh3_x64_128_sintdigest() -> None:
    assert mmh3.mmh3_x64_128_sintdigest(b"") == 0

    assert (
        mmh3.mmh3_x64_128_sintdigest(
            b"The quick brown fox jumps over the lazy dog", 0x9747B28C
        )
        == -8943985938913228316176695348732677855
    )


def test_mmh3_x64_128_uintdigest() -> None:
    assert mmh3.mmh3_x64_128_uintdigest(b"") == 0

    assert (
        mmh3.mmh3_x64_128_uintdigest(b"foo", 42)
        == 215966891540331383248189432718888555506
    )


def test_mmh3_x64_128_stupledigest() -> None:
    assert mmh3.mmh3_x64_128_stupledigest(b"") == (0, 0)

    assert mmh3.mmh3_x64_128_stupledigest(
        memoryview(b"The quick brown fox jumps over the lazy dog"), 0x9747B28C
    ) == (
        8325606756057297185,
        -484854449282476315,
    )


def test_mmh3_x64_128_utupledigest() -> None:
    assert mmh3.mmh3_x64_128_utupledigest(b"") == (0, 0)

    assert mmh3.mmh3_x64_128_utupledigest(memoryview(b"foo")) == (
        16316970633193145697,
        9128664383759220103,
    )


def test_mmh3_x86_128_digest() -> None:
    assert mmh3.mmh3_x86_128_digest(b"", 123) == (
        0x26F3E79926F3E79926F3E799FEDC5245
    ).to_bytes(16, "little")

    assert mmh3.mmh3_x86_128_digest(b"Hello, world!", 123) == (
        0x9E37C886A41621625A1AACD761C9129E
    ).to_bytes(16, "little")

    assert mmh3.mmh3_x86_128_digest(bytearray(b"Hello, world!"), 123) == (
        0x9E37C886A41621625A1AACD761C9129E
    ).to_bytes(16, "little")

    v = bytearray(b"hello, world!!!")
    mv = memoryview(v)
    v[0] = ord("H")

    assert mmh3.mmh3_x86_128_digest(mv[0:13], 123) == (
        0x9E37C886A41621625A1AACD761C9129E
    ).to_bytes(16, "little")


def test_mmh3_x86_128_sintdigest() -> None:
    assert mmh3.mmh3_x64_128_sintdigest(b"") == 0

    assert (
        mmh3.mmh3_x64_128_sintdigest(
            b"The quick brown fox jumps over the lazy dog", 0x9747B28C
        )
        == -8943985938913228316176695348732677855
    )


def test_mmh3_x86_128_uintdigest() -> None:
    assert mmh3.mmh3_x64_128_uintdigest(b"", 0) == 0

    # Test vector from https://github.com/PeterScott/murmur3/blob/master/test.c
    assert (
        mmh3.mmh3_x86_128_uintdigest(b"Hello, world!", 123)
        == 0x9E37C886A41621625A1AACD761C9129E
    )


def test_mmh3_x86_128_stupledigest() -> None:
    assert mmh3.mmh3_x86_128_stupledigest(b"", 0) == (0, 0)

    assert mmh3.mmh3_x86_128_stupledigest(
        memoryview(b"The quick brown fox jumps over the lazy dog"), 0x9747B28C
    ) == (
        5528275682885686622,
        -3623575540584727908,
    )


def test_mmh3_x86_128_utupledigest() -> None:
    assert mmh3.mmh3_x86_128_utupledigest(b"", 0) == (0, 0)

    # Test vector from https://github.com/PeterScott/murmur3/blob/master/test.c
    assert mmh3.mmh3_x86_128_utupledigest(memoryview(b"Hello, world!"), 123) == (
        0x5A1AACD761C9129E,
        0x9E37C886A4162162,
    )


def test_64bit() -> None:
    if sys.maxsize < (1 << 32):  # Skip this test under 32-bit environments
        return
    a = bytes(2**32 + 1)
    assert mmh3.hash(a) == -1710109261
    assert (
        mmh3.hash_bytes(a) == b"\x821\x93\x0c\xe7\xa8\x02\x9d\xe5 \xa6\xf9\xeb8\xd6\x0e"
    )


# from hex string "0xff00de" to integer
def hex_to_int(hex_str: str) -> int:
    return int(hex_str, 16)



# pylint: disable=missing-module-docstring,missing-function-docstring
from collections.abc import Callable
from concurrent.futures import ThreadPoolExecutor
from typing import Any

import mmh3


def run_threaded(func: Callable[..., Any], num_threads: int = 8) -> None:
    with ThreadPoolExecutor(max_workers=num_threads) as executor:
        futures = [executor.submit(func) for _ in range(num_threads)]
        for future in futures:
            future.result()  # wait for all threads to complete


def test_parallel_hasher_mmh3_32_update() -> None:
    hasher = mmh3.mmh3_32()

    def closure() -> None:
        for _ in range(1000):
            hasher.update(b"foo")

    run_threaded(closure, num_threads=8)

    assert hasher.sintdigest() == mmh3.hash(b"foo" * 8000)


def test_parallel_hasher_mmh3_x64_128_update() -> None:
    hasher = mmh3.mmh3_x64_128()

    def closure() -> None:
        for _ in range(1000):
            hasher.update(b"foo")

    run_threaded(closure, num_threads=8)

    assert hasher.sintdigest() == mmh3.hash128(b"foo" * 8000, x64arch=True, signed=True)


def test_parallel_hasher_mmh3_x86_128_update() -> None:
    hasher = mmh3.mmh3_x86_128()

    def closure() -> None:
        for _ in range(1000):
            hasher.update(b"foo")

    run_threaded(closure, num_threads=8)

    assert hasher.sintdigest() == mmh3.hash128(
        b"foo" * 8000, x64arch=False, signed=True
    )



# pylint: disable=missing-module-docstring, missing-function-docstring
# pylint: disable=no-value-for-parameter, too-many-function-args
from typing import no_type_check

import mmh3
import pytest


@no_type_check
def test_hash_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.hash()
    with pytest.raises(TypeError):
        mmh3.hash(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.hash(b"hello, world", 42, True, 1234)
    with pytest.raises(TypeError):
        mmh3.hash(b"hello, world", seed="42")
    with pytest.raises(TypeError):
        mmh3.hash([1, 2, 3], 42)
    # pylint: disable=redundant-keyword-arg
    with pytest.raises(TypeError):
        mmh3.hash(b"hello, world", key=b"42")


@no_type_check
def test_hash_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.hash(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.hash(b"hello, world", 2**32)


@no_type_check
def test_hash128_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.hash128()
    with pytest.raises(TypeError):
        mmh3.hash128(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.hash128(b"hello, world", 42, True, False, 1234)
    with pytest.raises(TypeError):
        mmh3.hash128(b"hello, world", seed="42")
    with pytest.raises(TypeError):
        mmh3.hash128([1, 2, 3], 42)


@no_type_check
def test_hash128_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.hash128(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.hash128(b"hello, world", 2**32)


@no_type_check
def test_hash64_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.hash64()
    with pytest.raises(TypeError):
        mmh3.hash64(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.hash64(b"hello, world", 42, True, False, 1234)
    with pytest.raises(TypeError):
        mmh3.hash64(b"hello, world", seed="42")
    with pytest.raises(TypeError):
        mmh3.hash64([1, 2, 3], 42)


@no_type_check
def test_hash64_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.hash64(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.hash64(b"hello, world", 2**32)


@no_type_check
def test_hash_bytes_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.hash_bytes()
    with pytest.raises(TypeError):
        mmh3.hash_bytes(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.hash_bytes(b"hello, world", 42, True, 1234)
    with pytest.raises(TypeError):
        mmh3.hash_bytes(b"hello, world", seed="42")
    with pytest.raises(TypeError):
        mmh3.hash_bytes([1, 2, 3], 42)


@no_type_check
def test_hash_bytes_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.hash_bytes(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.hash_bytes(b"hello, world", 2**32)


@no_type_check
def test_hash_from_buffer_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.hash_from_buffer()
    with pytest.raises(TypeError):
        mmh3.hash_from_buffer(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.hash_from_buffer(b"hello, world", 42, True, 1234)
    with pytest.raises(TypeError):
        mmh3.hash_from_buffer(b"hello, world", seed="42")
    with pytest.raises(TypeError):
        mmh3.hash_from_buffer([1, 2, 3], 42)


@no_type_check
def test_hash_from_buffer_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.hash_from_buffer(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.hash_from_buffer(b"hello, world", 2**32)


@no_type_check
def test_mmh3_32_digest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_32_digest()
    with pytest.raises(TypeError):
        mmh3.mmh3_32_digest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_32_digest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_digest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_digest([1, 2, 3], 42)


@no_type_check
def test_mmh3_32_digest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_32_digest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_32_digest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_32_sintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_32_sintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_32_sintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_32_sintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_sintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_sintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_32_sintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_32_sintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_32_sintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_32_uintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_32_uintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_32_uintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_32_uintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_uintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_32_uintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_32_uintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_32_uintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_32_uintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_digest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_digest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_digest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_digest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_digest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_digest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_digest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_digest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_digest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_sintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_sintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_sintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_sintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_sintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_sintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_sintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_sintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_sintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_uintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_uintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_uintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_uintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_uintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_uintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_uintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_uintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_uintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_stupledigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_stupledigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_stupledigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_stupledigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_stupledigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_stupledigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_stupledigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_stupledigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_stupledigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_utupledigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_utupledigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_utupledigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_utupledigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_utupledigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128_utupledigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_utupledigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_utupledigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128_utupledigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_digest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_digest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_digest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_digest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_digest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_digest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_digest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_digest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_digest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_sintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_sintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_sintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_sintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_sintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_sintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_sintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_sintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_sintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_uintdigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_uintdigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_uintdigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_uintdigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_uintdigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_uintdigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_uintdigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_uintdigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_uintdigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_stupledigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_stupledigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_stupledigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_stupledigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_stupledigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_stupledigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_stupledigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_stupledigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_stupledigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_utupledigest_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_utupledigest()
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_utupledigest(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_utupledigest("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_utupledigest(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128_utupledigest([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_utupledigest_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_utupledigest(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128_utupledigest(b"hello, world", 2**32)


@no_type_check
def test_mmh3_32_init_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_32(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_32("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_32(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_32([1, 2, 3], 42)


@no_type_check
def test_mmh3_32_init_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_32(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_32(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x64_128_init_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x64_128([1, 2, 3], 42)


@no_type_check
def test_mmh3_x64_128_init_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x64_128(b"hello, world", 2**32)


@no_type_check
def test_mmh3_x86_128_init_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128(b"hello, world", 42, 1234)
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128("hello, world")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128(b"hello, world", "42")
    with pytest.raises(TypeError):
        mmh3.mmh3_x86_128([1, 2, 3], 42)


@no_type_check
def test_mmh3_x86_128_init_raises_valueerror() -> None:
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128(b"hello, world", -1)
    with pytest.raises(ValueError):
        mmh3.mmh3_x86_128(b"hello, world", 2**32)
